/*----- PROTECTED REGION ID(EventListener.cpp) ENABLED START -----*/
//=============================================================================
//
// file :        EventListener.cpp
//
// description : C++ source for the EventListener class and its commands.
//               The class is derived from Device. It represents the
//               CORBA servant object which will be accessed from the
//               network. All commands which can be executed on the
//               EventListener are implemented in this file.
//
// project :     EventListener
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
//
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include <EventListener.h>
#include <EventListenerClass.h>

#include <EventCallBack.h>

/*----- PROTECTED REGION END -----*/	//	EventListener.cpp

/**
 *  EventListener class description:
 *    EventListener
 */

//================================================================
//  The following table gives the correspondence
//  between command and method names.
//
//  Command name  |  Method name
//================================================================
//  State         |  Inherited (no method)
//  Status        |  Inherited (no method)
//================================================================

//================================================================
//  Attributes managed is:
//================================================================
//================================================================

namespace EventListener_ns
{
/*----- PROTECTED REGION ID(EventListener::namespace_starting) ENABLED START -----*/

//	static initializations

/*----- PROTECTED REGION END -----*/	//	EventListener::namespace_starting

//--------------------------------------------------------
/**
 *	Method      : EventListener::EventListener()
 *	Description : Constructors for a Tango device
 *                implementing the classEventListener
 */
//--------------------------------------------------------
EventListener::EventListener(Tango::DeviceClass *cl, string &s)
 : TANGO_BASE_CLASS(cl, s.c_str())
{
	/*----- PROTECTED REGION ID(EventListener::constructor_1) ENABLED START -----*/
	init_device();
	
	/*----- PROTECTED REGION END -----*/	//	EventListener::constructor_1
}
//--------------------------------------------------------
EventListener::EventListener(Tango::DeviceClass *cl, const char *s)
 : TANGO_BASE_CLASS(cl, s)
{
	/*----- PROTECTED REGION ID(EventListener::constructor_2) ENABLED START -----*/
	init_device();
	
	/*----- PROTECTED REGION END -----*/	//	EventListener::constructor_2
}
//--------------------------------------------------------
EventListener::EventListener(Tango::DeviceClass *cl, const char *s, const char *d)
 : TANGO_BASE_CLASS(cl, s, d)
{
	/*----- PROTECTED REGION ID(EventListener::constructor_3) ENABLED START -----*/
	init_device();
	
	/*----- PROTECTED REGION END -----*/	//	EventListener::constructor_3
}

//--------------------------------------------------------
/**
 *	Method      : EventListener::delete_device()
 *	Description : will be called at device destruction or at init command
 */
//--------------------------------------------------------
void EventListener::delete_device()
{
	DEBUG_STREAM << "EventListener::delete_device() " << device_name << endl;
	/*----- PROTECTED REGION ID(EventListener::delete_device) ENABLED START -----*/
	
	//	Delete device allocated objects
	if(m_eventCallBack){
		delete m_eventCallBack;
		m_eventCallBack= 0;	
	}

	/*----- PROTECTED REGION END -----*/	//	EventListener::delete_device
}

//--------------------------------------------------------
/**
 *	Method      : EventListener::init_device()
 *	Description : will be called at device initialization.
 */
//--------------------------------------------------------
void EventListener::init_device()
{
	DEBUG_STREAM << "EventListener::init_device() create device " << device_name << endl;
	/*----- PROTECTED REGION ID(EventListener::init_device_before) ENABLED START -----*/
	
	//	Initialization before get_device_property() call
	
	/*----- PROTECTED REGION END -----*/	//	EventListener::init_device_before
	

	//	Get the device properties from database
	get_device_property();
	
	/*----- PROTECTED REGION ID(EventListener::init_device) ENABLED START -----*/
	
	//	Initialize device
	m_eventCallBack= new EventCallBack(this);

	bool stateless_flag= false;

	//Create proxy
	
	//== RX ==
	rx_proxy= 0;
	std::vector<std::string> rx_events {
		"rx_mode",
		"b1_capabilityState",
		"b2_capabilityState",
		"b3_capabilityState",
		"b4_capabilityState",
		"b5_capabilityState",
		"rx_mode_test",
		"configureBandStatus"
	};
	
	try{
		rx_proxy= new Tango::DeviceProxy(rx_proxy_name);
		for(unsigned int i=0;i<rx_events.size();i++){
			rx_proxy->subscribe_event(rx_events[i],Tango::PERIODIC_EVENT,m_eventCallBack,stateless_flag);
			rx_proxy->subscribe_event(rx_events[i],Tango::CHANGE_EVENT,m_eventCallBack,stateless_flag);
			rx_proxy->subscribe_event(rx_events[i],Tango::USER_EVENT,m_eventCallBack,stateless_flag);
		}
	}
	catch(Tango::DevFailed& e){
		Tango::Except::print_exception(e);
		set_state(Tango::FAULT);
		set_status("Failed to connect to proxy " + rx_proxy_name + " and subscribe to events!");
	}

	
	//== SPF ==
	spf_proxy= 0;
	std::vector<std::string> spf_events {
		"spf_mode",
		"b1_capabilityState",
		"b2_capabilityState",
		"b3_capabilityState",
		"b4_capabilityState",
		"b5_capabilityState",
		"spf_mode_test",
		"operateStatus"
	};

	try{
		spf_proxy= new Tango::DeviceProxy(spf_proxy_name);
		for(unsigned int i=0;i<spf_events.size();i++){
			spf_proxy->subscribe_event(spf_events[i],Tango::PERIODIC_EVENT,m_eventCallBack,stateless_flag);
			spf_proxy->subscribe_event(spf_events[i],Tango::CHANGE_EVENT,m_eventCallBack,stateless_flag);
			spf_proxy->subscribe_event(spf_events[i],Tango::USER_EVENT,m_eventCallBack,stateless_flag);
		}
	}
	catch(Tango::DevFailed& e){
		Tango::Except::print_exception(e);
		set_state(Tango::FAULT);
		set_status("Failed to connect to proxy " + spf_proxy_name + " and subscribe to events!");
	}


	//== DS ==
	ds_proxy= 0;
	std::vector<std::string> ds_events {
		"ds_mode",
		"ds_powerState",
		"ind_Position",
		"ds_mode_test",
		"ds_powerState_test"
	};

	try{
		ds_proxy= new Tango::DeviceProxy(ds_proxy_name);
		for(unsigned int i=0;i<ds_events.size();i++){
			ds_proxy->subscribe_event(ds_events[i],Tango::PERIODIC_EVENT,m_eventCallBack,stateless_flag);
			ds_proxy->subscribe_event(ds_events[i],Tango::CHANGE_EVENT,m_eventCallBack,stateless_flag);
			ds_proxy->subscribe_event(ds_events[i],Tango::USER_EVENT,m_eventCallBack,stateless_flag);
		}
	}
	catch(Tango::DevFailed& e){
		Tango::Except::print_exception(e);
		set_state(Tango::FAULT);
		set_status("Failed to connect to proxy " + ds_proxy_name + " and subscribe to events!");
	}

	
	/*----- PROTECTED REGION END -----*/	//	EventListener::init_device
}

//--------------------------------------------------------
/**
 *	Method      : EventListener::get_device_property()
 *	Description : Read database to initialize property data members.
 */
//--------------------------------------------------------
void EventListener::get_device_property()
{
	/*----- PROTECTED REGION ID(EventListener::get_device_property_before) ENABLED START -----*/
	
	//	Initialize property data members
	
	/*----- PROTECTED REGION END -----*/	//	EventListener::get_device_property_before


	//	Read device properties from database.
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("rx_proxy_name"));
	dev_prop.push_back(Tango::DbDatum("spf_proxy_name"));
	dev_prop.push_back(Tango::DbDatum("ds_proxy_name"));

	//	is there at least one property to be read ?
	if (dev_prop.size()>0)
	{
		//	Call database and extract values
		if (Tango::Util::instance()->_UseDb==true)
			get_db_device()->get_property(dev_prop);
	
		//	get instance on EventListenerClass to get class property
		Tango::DbDatum	def_prop, cl_prop;
		EventListenerClass	*ds_class =
			(static_cast<EventListenerClass *>(get_device_class()));
		int	i = -1;

		//	Try to initialize rx_proxy_name from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  rx_proxy_name;
		else {
			//	Try to initialize rx_proxy_name from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  rx_proxy_name;
		}
		//	And try to extract rx_proxy_name value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  rx_proxy_name;

		//	Try to initialize spf_proxy_name from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  spf_proxy_name;
		else {
			//	Try to initialize spf_proxy_name from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  spf_proxy_name;
		}
		//	And try to extract spf_proxy_name value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  spf_proxy_name;

		//	Try to initialize ds_proxy_name from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  ds_proxy_name;
		else {
			//	Try to initialize ds_proxy_name from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  ds_proxy_name;
		}
		//	And try to extract ds_proxy_name value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  ds_proxy_name;

	}

	/*----- PROTECTED REGION ID(EventListener::get_device_property_after) ENABLED START -----*/
	
	//	Check device property data members init
	
	/*----- PROTECTED REGION END -----*/	//	EventListener::get_device_property_after
}

//--------------------------------------------------------
/**
 *	Method      : EventListener::always_executed_hook()
 *	Description : method always executed before any command is executed
 */
//--------------------------------------------------------
void EventListener::always_executed_hook()
{
	DEBUG_STREAM << "EventListener::always_executed_hook()  " << device_name << endl;
	/*----- PROTECTED REGION ID(EventListener::always_executed_hook) ENABLED START -----*/
	
	//	code always executed before all requests
	
	/*----- PROTECTED REGION END -----*/	//	EventListener::always_executed_hook
}

//--------------------------------------------------------
/**
 *	Method      : EventListener::read_attr_hardware()
 *	Description : Hardware acquisition for attributes
 */
//--------------------------------------------------------
void EventListener::read_attr_hardware(TANGO_UNUSED(vector<long> &attr_list))
{
	DEBUG_STREAM << "EventListener::read_attr_hardware(vector<long> &attr_list) entering... " << endl;
	/*----- PROTECTED REGION ID(EventListener::read_attr_hardware) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EventListener::read_attr_hardware
}


//--------------------------------------------------------
/**
 *	Method      : EventListener::add_dynamic_attributes()
 *	Description : Create the dynamic attributes if any
 *                for specified device.
 */
//--------------------------------------------------------
void EventListener::add_dynamic_attributes()
{
	/*----- PROTECTED REGION ID(EventListener::add_dynamic_attributes) ENABLED START -----*/
	
	//	Add your own code to create and add dynamic attributes if any
	
	/*----- PROTECTED REGION END -----*/	//	EventListener::add_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Method      : EventListener::add_dynamic_commands()
 *	Description : Create the dynamic commands if any
 *                for specified device.
 */
//--------------------------------------------------------
void EventListener::add_dynamic_commands()
{
	/*----- PROTECTED REGION ID(EventListener::add_dynamic_commands) ENABLED START -----*/
	
	//	Add your own code to create and add dynamic commands if any
	
	/*----- PROTECTED REGION END -----*/	//	EventListener::add_dynamic_commands
}

/*----- PROTECTED REGION ID(EventListener::namespace_ending) ENABLED START -----*/

//	Additional Methods

/*----- PROTECTED REGION END -----*/	//	EventListener::namespace_ending
} //	namespace
